package com.sooka.sponest.monitor.dahua.controller;


import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.shaded.com.google.gson.Gson;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.sooka.sponest.monitor.camera.mapper.CentermonitorTCameraMapper;
import com.sooka.sponest.monitor.dahua.domain.CentermonitorTMonitoringEquipmentPositionDO;
import com.sooka.sponest.monitor.dahua.mapper.CentermonitorTMonitoringEquipmentPositionMapper;
import com.sooka.sponest.monitor.dahua.service.DahuaService;
import com.sooka.sponest.monitor.dahua.utils.HttpEnum;
import com.sooka.sponest.monitor.dahua.utils.HttpTestUtils;
import com.sooka.sponest.monitor.remoteapi.service.lawenforcement.RemoteLawenforcementBaseService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.springframework.web.bind.annotation.RequestMethod.GET;


/**
 * 对接大华根据组织获取执法记录仪、单兵设备在线状态
 */

@Slf4j
@Component
@Api(tags = "对接大华根据组织获取执法记录仪、单兵设备在线状态定时任务")
@RestController
@RequestMapping("/equipmentStatus/quartz")
public class EquipmentStatusTaskController {

    @Resource
    private CentermonitorTCameraMapper centermonitorTCameraMapper;
    @Resource
    private RemoteLawenforcementBaseService remoteLawenforcementBaseService;
    @Value("${sooka.dahua_interface_server.loginIp}")
    private String loginIp;
    @Value("${sooka.dahua_interface_server.loginPort}")
    private String loginPort;
    @Value("${sooka.dahua_interface_server.userName}")
    private String userName;
    @Value("${sooka.dahua_interface_server.userPwd}")
    private String userPwd;
    //    private String loginIp = "10.53.0.35";
//    private String loginPort = "7901";
//    private String userName = "system";
//    private String userPwd = "Admin123";
    //按组织获取设备详细信息
    public static final String ACTION = "/videoService/devicesManager/devicesInfo";
    private List<String> orgCodes = Arrays.asList("11033445593778368", "11248668755298496");
    @Resource
    DahuaService dahuaService;
    @Resource
    CentermonitorTMonitoringEquipmentPositionMapper centermonitorTMonitoringEquipmentPositionMapper;

    //    @Scheduled(cron = "0/5 * * * * ?")
    @Log(title = "对接大华根据组织获取执法记录仪、单兵设备在线状态定时任务", businessType = BusinessType.OTHER)
    @ApiOperation(value = "对接大华根据组织获取执法记录仪、单兵设备在线状态定时任务", notes = "对接大华根据组织获取执法记录仪、单兵设备在线状态定时任务")
    @RequestMapping(value = "/getLongPollingPositionMsg", method = GET)
    public void getLongPollingPositionMsg() throws Exception {
        log.info("开始执行获取记录仪在离线数据定时任务");
        R<?> result = HttpTestUtils.getToken(loginIp, Integer.parseInt(loginPort), userName, userPwd);
        if (result.getCode() != 200) {
            log.info("获取token失败，请检查配置信息:", JSONObject.parseObject(result.getMsg()));
        }
        String token = result.getData().toString();
        getMessage(loginIp, Integer.parseInt(loginPort), token);
    }

    /**
     * 根据组织code编码获取执法仪详情数据
     *
     * @param ip
     * @param port
     * @param token
     */
    private void getMessage(String ip, int port, String token) {
        Map<String, String> content = new HashMap<>();
        for (String orgCode : orgCodes) {
            content.put("orgCode", orgCode);
            getDevicesInfo(ip, port, token, content, orgCode);
        }
    }

    private void getDevicesInfo(String ip, int port, String token, Map<String, String> content, String orgCode) {
        String response = HttpTestUtils.httpRequest(HttpEnum.POST, ip, port, ACTION, token, new Gson().toJson(content));
        JSONObject jsonObject = JSONObject.parseObject(response);
        if (jsonObject.get("devices") == null) {
            log.info("根据组织code编码获取执法仪response:{}", response);
            return;
        }
        Map<String, Object> rsp = new Gson().fromJson(response, Map.class);
        List<Map<String, Object>> arr = (List<Map<String, Object>>) rsp.get("devices");
        log.info("获取记录仪在离线数据 size = {}", arr.size());
        updateDeviceState(arr, orgCode);
    }

    /**
     * 更新执法仪在离线状态
     *
     * @param arr
     */
    private void updateDeviceState(List<Map<String, Object>> arr, String orgCode) {
        log.info("根据组织code编码获取执法仪详情数据:{}", arr.size());
        if (CollectionUtils.isNotEmpty(arr)) {
            if (orgCode.equals("11248668755298496")) {
                List<Map<String, Object>> list = (List<Map<String, Object>>) arr.get(0).get("units");
                for (Map<String, Object> map : list) {
                    if (map.get("unitType").toString().equals("1")) {
                        arr = (List<Map<String, Object>>) map.get("channels");
                        //更新摄像头类型为录像机在离线状态
                        List<String> channelCodeAllList = arr.stream().map(item -> item.get("channelCode"))
                                .filter(channelCode -> channelCode != null)
                                .map(Object::toString)
                                .collect(Collectors.toList());
                        //先把摄像头类型为录像机的状态全部改成离线
                        if (CollectionUtils.isNotEmpty(channelCodeAllList)) {
                            centermonitorTCameraMapper.updateBatchVideoRecorderOfflineState(channelCodeAllList);
                        }
                        //再修改摄像头类型为录像机的在线状态
                        List<String> channelCodeList = arr.stream()
                                .filter(item -> "1".equals(item.get("status")))
                                .map(item -> item.get("channelCode"))
                                .filter(channelCode -> channelCode != null)
                                .map(Object::toString)
                                .collect(Collectors.toList());
                        if (CollectionUtils.isNotEmpty(channelCodeList)) {
                            centermonitorTCameraMapper.updateBatchVideoRecorderOnlineState(channelCodeList);
                        }
                        break;
                    }
                }
            } else {
                //向执法中心推送设备状态
                R result = remoteLawenforcementBaseService.deviceOpenOrClose(arr);
                if (result.getCode() != 200) {
                    log.info("向执法中心推送设备状态失败:{}", result.getMsg());
                }
                //更新执法仪在离线状态
                centermonitorTCameraMapper.updateBatchLawEnforcementInstrumentState(arr);
            }
        }
    }

    /**
     *获取记录仪位置数据
     */
    @PostMapping("putDahuaRecorderLocation")
    @ResponseBody
    public AjaxResult insertLocationListToday(@RequestBody CentermonitorTMonitoringEquipmentPositionDO centermonitorTMonitoringEquipmentPositionDO) {
        try {
            dahuaService.insertEquipmentPosition(centermonitorTMonitoringEquipmentPositionDO);
            //更新执法记录仪经纬度
            log.info("获取记录仪位置数据:{}",centermonitorTMonitoringEquipmentPositionDO);
            centermonitorTMonitoringEquipmentPositionMapper.updateLongitudeAndLatitudeByCameraCode(centermonitorTMonitoringEquipmentPositionDO.getGpsX(),centermonitorTMonitoringEquipmentPositionDO.getGpsY(),centermonitorTMonitoringEquipmentPositionDO.getChannelCode());
            return AjaxResult.success();
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxResult.error(500, "内部服务器错误");
        }
    }

}
